#
# fishing.R, 28 Feb 21
#
# Data from:
# ARRAY LAYOUTS FOR COMPARISON-BASED SEARCHING
# Paul-Virak Khuong and Pat Morin

library("colorspace")
library("plyr")

# Data from one processor
ad=read.csv("atom_D2700.csv", as.is=TRUE, sep=" ")

ad=subset(ad, alg != "fake")

plot(ad$N, ad$run_time, log="x")

# Look at 32-bit data
a32=subset(ad, (a_type=="uint32"))
sorted_32=subset(a32, (alg=="sorted"))

# Plot results for 32-bit array datatype
alg_line=function(df)
{
lines(df$N, df$run_time, col=df$col_str[1])
}

par(las=1)
par(bty="l")

plot(0.5, type="n", log="x",
     	xaxs="i",
     	xlim=range(a32$N), ylim=range(a32$run_time),
	xlab="Array elements", ylab="Iteration time (seconds)")

u_alg=unique(a32$alg)
pal_col=rainbow(length(u_alg))
a32$col_str=mapvalues(a32$alg, u_alg, pal_col)
dum=ddply(a32, .(alg), alg_line)


# One approach
library("rdrobust")

rdat_mod=rdrobust(sorted_32$run_time, log(sorted_32$N), c=log(2^17), kernel = "triangular", p = 2)

summary(rdat_mod)


# Another package
library("rdd")

ddat_mod=RDestimate(run_time ~ log(N)+alg, data=a32, cutpoint=log(2^17))
summary(ddat_mod)


# This gave the best fit
library("segmented")

# Normalise runtime by algorithm
norm_alg=function(df)
{
df$norm_rt=df$run_time/max(df$run_time)

return(df)
}


a32$log_N=log(a32$N)
a32=ddply(a32, .(alg), norm_alg)
sorted_32=subset(a32, (alg=="sorted"))

plot(sorted_32$log_N, sorted_32$norm_rt)

sat_mod<-glm(norm_rt ~ log_N, family=binomial, data=sorted_32)
sat_seg<-segmented(sat_mod, seg.Z=~log_N, psi=log(2^17))
summary(sat_seg)

at_mod<-glm(norm_rt ~ log_N+I(log_N^2)+log_N:alg, family=binomial, data=a32)
at_seg<-segmented(at_mod, seg.Z=~log_N, psi=log(2^17))
summary(at_seg)

slope(at_seg)

